#include "dx_run.h"

const int D3DFVF_VERTEX2D = D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1;

LPDIRECT3D9 d3d;
D3DPRESENT_PARAMETERS d3dpp;

//---------------------------------------------------------------------------
void InitD3D()
{
	d3d = Direct3DCreate9(D3D_SDK_VERSION);
	ReinitDXParameters();
}
//---------------------------------------------------------------------------
void ReinitDXParameters()
{
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.Windowed = true;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;
}
//---------------------------------------------------------------------------
void CreateDXSurface(LPDIRECT3DDEVICE9 &surface, HWND h, int wndWidth, int wndHeight)
{
	d3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, h, D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &surface);
	ResizeDXSurface(surface, wndWidth, wndHeight);
}
//---------------------------------------------------------------------------
 void ResizeDXSurface(LPDIRECT3DDEVICE9 &surface, int newWidth, int newHeight)
{
	D3DXMATRIX projectionMatrix;
	D3DXMatrixOrthoLH(&projectionMatrix, newWidth, newHeight, 1.0f, 100.0f);
	surface->SetTransform(D3DTS_PROJECTION, &projectionMatrix);
}
//---------------------------------------------------------------------------
void FreeD3D()
{
		if (d3d != NULL) d3d->Release();
}
//---------------------------------------------------------------------------
D3DCOLOR RetDXRGBColor(char r, char g, char b)
{
	return D3DCOLOR_XRGB(r, g, b);
};
//---------------------------------------------------------------------------
/* SVertex2D */
//---------------------------------------------------------------------------
void SVertex2D::Set(float aX, float aY, float aZ, float TextureX, float TextureY, DWORD Color)
{
	x = aX;
	y = aY;
	z = aZ;
	rhw = 1.0f;
	color = Color;
	textureX = TextureX;
	textureY = TextureY;
}
//---------------------------------------------------------------------------
/* end of SVertex2D */

/* SQuad */
//---------------------------------------------------------------------------
SQuad::SQuad(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z)
{
	Set(x1, y1, x2, y2, x3, y3, x4, y4, z);
}
//---------------------------------------------------------------------------
void SQuad::Set(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z)
{
	vertices[0].Set(x4, y4, z, 0.0f, 1.0f);
	vertices[1].Set(x1, y1, z, 0.0f, 0.0f);
	vertices[2].Set(x3, y3, z, 1.0f, 1.0f);
	vertices[3].Set(x2, y2, z, 1.0f, 0.0f);
}
//---------------------------------------------------------------------------
void SQuad::SetPos(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z)
{
	vertices[0].SetPos(x4, y4);
	vertices[1].SetPos(x1, y1);
	vertices[2].SetPos(x3, y3);
	vertices[3].SetPos(x2, y2);
}
//---------------------------------------------------------------------------
void SQuad::SetTexCoords(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4)
{
	vertices[0].SetTexCoords(x4, y4);
	vertices[1].SetTexCoords(x1, y1);
	vertices[2].SetTexCoords(x3, y3);
	vertices[3].SetTexCoords(x2, y2);
}
//---------------------------------------------------------------------------
/* end of SQuad */

/* CTexture */
//---------------------------------------------------------------------------
void CTexture::ReleaseTexture()
{
	if (texture != NULL) texture->Release();
}
//---------------------------------------------------------------------------
void CTexture::Reinit()
{
	ReleaseTexture();
	LoadTextureFromFile(fName, surface);
}
//---------------------------------------------------------------------------
void CTexture::LoadTextureFromFile(std::string fileName, LPDIRECT3DDEVICE9 &Surface)
{
	D3DXIMAGE_INFO imageInfo;

	D3DXCreateTextureFromFileEx(Surface, fileName.c_str(), NULL, NULL, NULL, 0, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_FILTER_NONE, D3DX_FILTER_NONE, 0, &imageInfo, NULL, &texture);
	width = imageInfo.Width;
	height = imageInfo.Height;
}
//---------------------------------------------------------------------------
CTexture::CTexture(std::string fileName, LPDIRECT3DDEVICE9 &Surface)
{
	fName = fileName;
	surface = Surface;
	LoadTextureFromFile(fName, surface);
}
//---------------------------------------------------------------------------
CTexture::~CTexture()
{
	ReleaseTexture();
}
//---------------------------------------------------------------------------
/* end of CTexture */