#ifndef DX_RUN_H_INCLUDED
#define DX_RUN_H_INCLUDED

#include <windows.h>
#include <vector>
#include <string>
#include <d3d9.h>
#include <d3dx9.h>

extern const int D3DFVF_VERTEX2D;

void InitD3D();
void ReinitDXParameters();
void CreateDXSurface(LPDIRECT3DDEVICE9 &surface, HWND h, int wndWidth, int wndHeight);
void ResizeDXSurface(LPDIRECT3DDEVICE9 &surface, int newWidth, int newHeight);
void FreeD3D();
D3DCOLOR RetDXRGBColor(char r, char g, char b);

struct SVertex2D
{
	float x, y, z;
	float rhw;
	DWORD color;
	float textureX, textureY;
	void Set(float aX, float aY, float aZ, float TextureX, float TextureY, DWORD Color = 0xffffffff);
	void SetPos(float aX, float aY, float aZ = 1.0f) { x = aX; y = aY; z = aZ; };
	void SetTexCoords(float TextureX, float TextureY) { textureX = TextureX; textureY = TextureY; };
	SVertex2D() {};
	SVertex2D(float aX, float aY, float aZ, float TextureX, float TextureY, DWORD Color = 0xffffffff) { Set(aX, aY, aZ, TextureX, TextureY, Color); }
};

struct SQuad
{
	public :
		SVertex2D vertices[4];
		void Set(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z = 1.0f);
		void SetPos(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z = 1.0f);
		void SetTexCoords(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4);
		SQuad() {};
		SQuad(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float z = 1.0f);
};

class CTexture
{
	private:
		int width, height;
		std::string fName;
		LPDIRECT3DDEVICE9 surface;
		LPDIRECT3DTEXTURE9 texture;
		void ReleaseTexture();
	public:
		const LPDIRECT3DTEXTURE9 &GetTexture() { return texture; };
		int GetWidth() { return width; };
		int GetHeight() { return height; };
		void Reinit();
		void LoadTextureFromFile(std::string fileName, LPDIRECT3DDEVICE9 &Surface);
		CTexture(std::string fileName, LPDIRECT3DDEVICE9 &Surface);
		~CTexture();
};

#endif