#include "findway_algo.h"
#include <windows.h>
#include <iostream>
#include <fstream>
#include <math.h>
#include <time.h>
#include <stdlib.h> // dla rand

const int UNSEEN_FIELD = -1; // warto dla niedowiedzonego pola
const DIRECTIONS_COUNT = 8; // ilo elementw tablicy z kierunkami
const SPoint nearPoints[DIRECTIONS_COUNT] = {SPoint(0, -1), SPoint(1, 0), SPoint(0, 1), SPoint(-1, 0),
															SPoint(-1, -1), SPoint(1, -1), SPoint(1, 1), SPoint(-1, 1)};

int StrToInt(std::string str) 
{
  std::istringstream istr(str) ;
  int num ;
  if (istr >> num)
    return num ;
  else return 0 ;
}

std::string IntToStr(int num) 
{
  std::ostringstream ostr ;
  if (ostr << num)
    return ostr.str() ;
  else return "" ;
}

std::string FloatToStr(double num)
{
	std::ostringstream ostr ;
	if (ostr << num)
		return ostr.str() ;
	else return "" ;
}

/* CMap */
//---------------------------------------------------------------------------
CMap::CMap()
{
	mapSize.Set(0, 0);
}
//---------------------------------------------------------------------------
CMap::CMap(int mapWidth, int mapHeight, bool random, bool saveToFile, int wallsPercent)
{
	Resize(mapWidth, mapHeight);

	if (random)
	{
		int value;
		srand(time(NULL));
		for (int i = 0; i < mapSize.Y; i++)
		{
			for (int j = 0; j < mapSize.X; j++)
			{
				value = rand() % 100;
				if (value < wallsPercent) map[i][j] = mtWall;
				else map[i][j] = mtFree;
			}
		}
		map[0][0] = mtStart;
		map[mapSize.Y - 1][mapSize.X - 1] = mtEnd;
		
		startPos.Set(0, 0);
		endPos.Set(mapSize.X - 1, mapSize.Y - 1);

		if (saveToFile)
		{
			std::ofstream ofstr("mapy\\losowa.txt", std::ios::out);
			ofstr << mapSize.X << " " << mapSize.Y << std::endl;

			for (int i = 0; i < mapSize.Y; i++)
			{
				for (int j = 0; j < mapSize.X; j++)
				{
					ofstr << IntToStr(map[i][j]);
				}
				ofstr << std::endl;
			}
			ofstr.close();
		}
	}
}
//---------------------------------------------------------------------------
CMap::CMap(std::string fileName)
{
	LoadMap(fileName);
}
//---------------------------------------------------------------------------
CMap::~CMap()
{
	Clear();
}
//---------------------------------------------------------------------------
void CMap::Clear()
{
	for (int i = 0; i < mapSize.Y; i++) map[i].clear();
	map.clear();
}
//---------------------------------------------------------------------------
void CMap::Resize(int newWidth, int newHeight)
{
	if ((mapSize.X == newWidth) && (mapSize.Y == newHeight)) return;
	map.resize(newHeight);
	for (int i = 0; i < newHeight; i++)
		map[i].resize(newWidth);
	
	mapSize.X = newWidth;
	mapSize.Y = newHeight;
}
//---------------------------------------------------------------------------
void CMap::LoadMap(std::string fileName)
{
	std::ifstream ifstr(fileName.c_str(), std::ios::in);
	int mapWidth, mapHeight;
	ifstr >> mapWidth >> mapHeight;
	Resize(mapWidth, mapHeight);

	int x = 0, y = 0, fieldType;
	char c;
	while (ifstr.get(c))
	{
		if (c == '\n') continue;
		fieldType = StrToInt(std::string(1, c));
		
		if (fieldType == mtStart)
		{
			startPos.X = x;
			startPos.Y = y;
		}
		else if (fieldType == mtEnd)
		{
			endPos.X = x;
			endPos.Y = y;
		}
	
		map[y][x] = fieldType;

		x++;
		if (x == mapSize.X)
		{
			x = 0;
			y++;
		}
	}

	ifstr.close();
}
//---------------------------------------------------------------------------
/* end of CMap */

/* CFindWayMap */
//---------------------------------------------------------------------------
void CFindWayMap::PrepareMap()
{
	for (int i = 0; i < mapSize.Y; i++)
		for (int j = 0; j < mapSize.X; j++)
			map[i][j] = UNSEEN_FIELD;
	bufWay.clear();
}
//---------------------------------------------------------------------------
void CFindWayMap::CreateWay(TWay *way, SPoint actualEndPos)
{
	TWay retWay;
	SPoint nextW;

	// dopki rozpatrywany punkt nie jest pole startowym wykonujemy:
	while (actualEndPos != startPos)
	{
		// - dodajemy aktualnie znaleziony punkt do pomocniczej tablicy drogi
		retWay.push_back(actualEndPos);
		// - okramy go w poniszej ptli, szukajc pola,
		// ktra ma warto o 1 mniejsz ni actualEndPos
		for (int i = 0; i < DIRECTIONS_COUNT; i++)
		{
			nextW.X = actualEndPos.X + nearPoints[i].X;
			nextW.Y = actualEndPos.Y + nearPoints[i].Y;
			if (IsPointOutOfMap(nextW)) continue;
			if (map[nextW.Y][nextW.X] == map[actualEndPos.Y][actualEndPos.X] - 1)
			{
				actualEndPos = nextW;
				break;
			}
		}
	}
	// poniszy if jest dodatkiem usprawniajcym algorytm
	// jeeli punkt docelowy jest nieosigalny
	if (((*way).size() > retWay.size()) || ((*way).size() == 0))
		(*way) = retWay;
}
//---------------------------------------------------------------------------
bool CFindWayMap::IsPointOutOfMap(SPoint point)
{
	if ((point.X < 0) || (point.X >= mapSize.X) || (point.Y < 0) || (point.Y >= mapSize.Y)) return true;
	return false;
}
//---------------------------------------------------------------------------
/*
Dane: Labirynt (u nas mapa wiata 2D, czyli prostokt z jednym wyjciem, wypeniony cianami, 
ktre s rwnolege do zewntrznych cian i nie tworz zamknitych obszarw. 
Dany jest punkt w polu s (nasz punkt startu)

Wynik: Droga w labiryncie, ktra prowadzi z pola s do wyjcia

Krok 0: Przyjmij, ze na pocztku nie bye w adnym polu labiryntu, 
a wic wszystkie pola s nieodwiedzone. 
Krok 1: Umie w kolejce Q pole s. W polu s umie liczb 0 (zero). 
Krok 2: Dopki kolejka Q nie jest pusta wykonuj kroki 3- 5 
Krok 3: Usu z kolejki Q jej pierwszy element - oznaczmy to pole przez v 
Krok 4: Dla kadego pola w ssiedniego wzgldem v i nie oddzielonego od niego cian wykonaj krok 5 
Krok 5: Jeli nie bye jeszcze w polu w, to umie w nim liczb o jeden wiesza od liczby 
znajdujcej si w polu v. Jeli pole w zawiera wyjcie to przejd do kroku 6, 
a w przeciwnym razie docz w do koca kolejki Q 
Krok 6: {W tym kroku budujemy od koca list pl tworzcych najkrtsz drog z pola s do pola w, 
na ktrym zakoczy dziaanie krok 5}
Dopki w nie jest polem s, wykonuj: za kolejne (od koca) pole drogi przyjmij w i za now warto w przyjmij ssiednie wzgldem w, w ktrym znajduje si liczba o jeden mniejsza od liczby znajdujcej si w obecnym polu w.
*/
void CFindWayMap::FindWay(SPoint startPoint, SPoint endPoint, TWay *way, const TMap &aMap)
{
	startPos = startPoint;
	endPos = endPoint;

	if (startPos == endPos) return;

	Resize(aMap[0].size(), aMap.size());
	PrepareMap(); // krok 0. - wszystkim polom przypisywana jest warto FIELD_UNSEEN = -1

	SPoint v, w;

	Q.push(startPos); // krok 1.
	map[startPos.Y][startPos.X] = 0; // krok 1.

	while (!Q.empty()) // krok 2.
	{
		v = Q.front(); // krok 3.
		Q.pop();

		for (int i = 0; i < DIRECTIONS_COUNT; i++) // krok 4 - okramy kolejne pola v
		{
			w.X = v.X + nearPoints[i].X;
			w.Y = v.Y + nearPoints[i].Y;
			if (IsPointOutOfMap(w)) continue;
			// krok 5. - jeli pole nie jest cian i jeszcze go nie odwiedzilimy, to:
			// - nadajemy mu warto o jeden wiksz ni posiada pole v
			// - dodajemy pole w na koniec kolejki
			if ((aMap[w.Y][w.X] != mtWall) && (map[w.Y][w.X] == UNSEEN_FIELD))
			{
				// ten if poniej to dodatek, eby droga nie przechodzia z kafla na kafel po skosie,
				// jeli kafle obok nie s puste
				/* 0 - wolne przejcie
				   3 - ciana
					 1 - aktualne pooenie
					 2 - na to miejsce z miejsca 1 nie mona przej, poniewa jest ono na skos,
					     a nad nim i z boku znajduje si ciana
							000
							013
							032
				*/
			  //if ((nearPoints[i].X == 0) || (nearPoints[i].Y == 0) || ((aMap[v.Y + nearPoints[i].Y][v.X] == 0) && (aMap[v.Y][v.X + nearPoints[i].X] == 0)))
				//{
					map[w.Y][w.X] = map[v.Y][v.X] + 1;
					Q.push(w);
				//}
			}
			if ((w.X == endPos.X) && (w.Y == endPos.Y))
			{
				CreateWay(&bufWay, w); // krok 6. - doszlimy do koca, tworzymy drog
				goto wayFound;
			} 
		}
	}
	wayFound:

	// tutaj zaczyna si cz wymylona przeze mnie
	// nie znaleziono drogi, tzn., e punkt docelowy jest np. ograniczony murem
	// musimy znale punkty najbliej oddalone od celu, do ktrych moemy dotrze,
	// i wybra ten, do ktrego droga jest najkrtsza
	if (bufWay.size() == 0) 
	{
		std::vector<SExtPoint> closestAvailablePoints;
		int distance;

		// sprawdzamy wszystkie pola i jeeli:
		for (int i = mapSize.Y - 1; i >= 0; i--)
			for (int j = mapSize.X -1; j >= 0; j--)
			{
				if ((aMap[i][j] != 0) || (map[i][j] == UNSEEN_FIELD) || ((j == endPos.X) && (i == endPos.Y))) continue;
				distance = abs(endPos.X - j) + abs(endPos.Y - i);
				// tablica z punktami jest pusta  dodajemy do niej aktualnie rozpatrywany punkt,
				if (closestAvailablePoints.size() == 0) closestAvailablePoints.push_back(SExtPoint(j, i, distance));
				else
				{
					// punkty w tablicy s tak samo oddalone jak aktualnie rozpatrywany punkt  dodajemy go na koniec,
					if (distance == closestAvailablePoints[0].lengthToEndPoint) closestAvailablePoints.push_back(SExtPoint(j, i, distance));
					else if (distance < closestAvailablePoints[0].lengthToEndPoint)
					{
				    // punkty w tablicy s pooone dalej  usuwamy wszystkie punkty z tablicy i dodajemy do niej aktualny punkt.
						closestAvailablePoints.clear();
						closestAvailablePoints.push_back(SExtPoint(j, i, distance));
					}
				}
			}

		// wywoujemy funkcj CreateWay dla kadego ze znalezionych punktw
		for (int i = 0; i < closestAvailablePoints.size(); i++)
			CreateWay(&bufWay, closestAvailablePoints[i]);
	}

	// teraz wypeniamy przesan tablic drogi punktami znajdujcymi si w pomocniczej
	// tablicy drogi, z tym, e punkty pobieramy od koca (bo droga bya tworzona od koca)
	for (std::vector<SPoint>::reverse_iterator it = bufWay.rbegin(); it != bufWay.rend(); it++)
		(*way).push_back(*it);
}
//---------------------------------------------------------------------------
CFindWayMap::CFindWayMap()
{
	log = new CILog(std::string("logFile.txt"), std::string("Wyszukiwanie drogi - log klasy CFindWayMap"));
}
//---------------------------------------------------------------------------
CFindWayMap::~CFindWayMap()
{
	if (log != NULL) delete log;
	log = NULL;
}
//---------------------------------------------------------------------------
/* end of CFindWayMap */