#ifndef FINDWAY_ALGO_H_INCLUDED
#define FINDWAY_ALGO_H_INCLUDED

#include <vector>
#include <string>
#include <queue>
#include <sstream>
#include "ilog.h"

enum mapTileType {mtFree = 0, mtStart = 1, mtEnd = 2, mtWall = 3, mtWay = 4};

std::string IntToStr(int num);
std::string FloatToStr(double num);

struct SPoint
{
	int X, Y;
	bool operator!=(SPoint arg) { if ((X != arg.X) || (Y != arg.Y)) return true; return false; };
	bool operator==(SPoint arg) { if ((X != arg.X) || (Y != arg.Y)) return false; return true; };
	void Set(int aX, int aY) { X = aX; Y = aY; };
	SPoint() {};
	SPoint(int aX, int aY) { Set(aX, aY); };
};

struct SExtPoint : public SPoint
{
	int lengthToEndPoint;
	SExtPoint(int aX, int aY, int length) { Set(aX, aY); lengthToEndPoint = length; };
	SExtPoint(SPoint point, int length) { Set(point.X, point.Y); lengthToEndPoint = length; };
};

typedef std::vector<int> TMapRow ;
typedef std::vector<TMapRow> TMap;
typedef std::vector<SPoint> TWay;

class CMap
{
	public:
		TMap map;
		SPoint mapSize; // rozmiar caej mapy (jednostka to kafle)
		SPoint startPos, endPos; // pozycje startowego i docelowego kafla
		void Clear();
		void Resize(int newWidth, int newHeight);
		void LoadMap(std::string fileName);

		CMap();
		CMap(int mapWidth, int mapHeight, bool random = false, bool saveToFile = false, int wallsPercent = 33);
		CMap(std::string fileName);
		~CMap();
};

class CFindWayMap : public CMap
{
	private:
		CILog *log;
		std::queue<SPoint> Q;
		TWay bufWay; 

		void PrepareMap();
		void CreateWay(TWay *way, SPoint endPos);
		bool IsPointOutOfMap(SPoint point);

	public:
		void FindWay(SPoint startPoint, SPoint endPoint, TWay *way, const TMap &aMap);
		CFindWayMap();
		~CFindWayMap();
};


#endif