#include "ilog.h"
#include <windows.h>

using namespace std;

enum TLogMessages { lmTimerNotSupported };
const std::string logMessages[] = { "High frequency timer is not supported on this machine" };
const std::string logError = " !!! --> ";

/* CILog */
//---------------------------------------------------------------------------
void CILog::WriteInt(int value, bool addSeparator)
{
	outStr << value;
	if (addSeparator) outStr << " ";
}
//---------------------------------------------------------------------------
void CILog::WriteString(const std::string &value, bool addSeparator)
{
	outStr << value;
	if (addSeparator) outStr << " ";
}
//---------------------------------------------------------------------------
void CILog::WriteDouble(double value, bool addSeparator)
{
	outStr << value;
	if (addSeparator) outStr << " ";
}
//---------------------------------------------------------------------------
void CILog::WritelnInt(int value)
{
	outStr << endl << value;
}
//---------------------------------------------------------------------------
void CILog::WritelnString(const std::string &value)
{
	outStr << endl << value;
}
//---------------------------------------------------------------------------
void CILog::WritelnDouble(double value)
{
	outStr << endl << value;
}
//---------------------------------------------------------------------------
void CILog::AddSeparator()
{
	outStr << "//---------------------------------------------------------------------------\n";
}
//---------------------------------------------------------------------------
void CILog::AddSeparator2()
{
	Add2Lines();
	AddSeparator();
	Add2Lines();
}
//---------------------------------------------------------------------------
void CILog::AddLine()
{
	outStr << endl;
}
//---------------------------------------------------------------------------
void CILog::Add2Lines()
{
	outStr << "\n\n";
}
//---------------------------------------------------------------------------
void CILog::StartCountingTime()
{
	QueryPerformanceFrequency((LARGE_INTEGER*)&frequency);
	QueryPerformanceCounter((LARGE_INTEGER*)&startTime);
}
//---------------------------------------------------------------------------
double CILog::EndCountingTime()
{
	QueryPerformanceCounter((LARGE_INTEGER*)&endTime);
	if (frequency == 0)
	{
		WritelnString(logError + logMessages[lmTimerNotSupported]);
		return -1;
	}
	return (endTime - startTime) * 1000 / frequency;
}
//---------------------------------------------------------------------------
void CILog::NewLog(std::string fileName)
{
	if (fileOpened) outStr.close();
	outStr.open(fileName.c_str(), ios::out);
	outStr << logHeader;
	AddLine();
	AddSeparator();
	fileOpened = true;
}
//---------------------------------------------------------------------------
CILog::CILog(std::string fileName, std::string aLogHeader)
{
	fileOpened = false;
	logHeader = aLogHeader;
	NewLog(fileName);
};
//---------------------------------------------------------------------------
CILog::~CILog()
{
	outStr.close();
}
//---------------------------------------------------------------------------
/* end of CILog */