#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>
#include "dx_run.h"
#include "findway_algo.h"

///// STAE
const height = 600 ;	// wysoko okna
const width = 800 ;		// szeroko
const mapTileTypesCount = 5;

///// ZMIENNE GLOBALNE
HWND hwnd ;					// uchwyt okna
HINSTANCE hInst ;		// instancja naszej aplikacji
bool keys[256];

CMap *map;
CFindWayMap *findWayMap;
TWay way;

int mapWidth, mapHeight;
LPDIRECT3DDEVICE9 mapSurface;
SQuad quad;
CTexture *mapTiles[mapTileTypesCount];
LPD3DXFONT dxFont;

void CreateObjects();
void ReleaseObjects();
void Draw();

void PrintText(int x, int y, std::string text, D3DCOLOR color)
{
	RECT r;
	r.left = x;
	r.top = y;
	r.right = x + 150;
	r.bottom = y + 100;

	dxFont->DrawText(NULL, text.c_str(), -1, &r, 0, color);
}
//---------------------------------------------------------------------------
void CreateObjects()
{
	CreateDXSurface(mapSurface, hwnd, width, height);
	mapTiles[mtFree] = new CTexture("grafika\\wolne.bmp", mapSurface);
	mapTiles[mtStart] = new CTexture("grafika\\start.bmp", mapSurface);
	mapTiles[mtEnd] = new CTexture("grafika\\koniec.bmp", mapSurface);
	mapTiles[mtWall] = new CTexture("grafika\\sciana.bmp", mapSurface);
	mapTiles[mtWay] = new CTexture("grafika\\droga.bmp", mapSurface);

	//map = new CMap("mapy\\mapa1.txt");
	map = new CMap(128, 128, true, false, 30); // tworzenie losowej mapy
	mapWidth = map->mapSize.X;
	mapHeight = map->mapSize.Y;

	findWayMap = new CFindWayMap();

	D3DXCreateFont(mapSurface, 15, 0, 0, 0, 0, 0, 0, 0, 0, "Arial", &dxFont);
}
//---------------------------------------------------------------------------
void ReleaseObjects()
{
	for (int i = 0; i < mapTileTypesCount; i++)
	{
		if (mapTiles[i] != NULL) delete mapTiles[i];
		mapTiles[i] = NULL;
	}
	if (findWayMap != NULL) delete findWayMap;
	findWayMap = NULL;
	
	if (findWayMap != NULL) delete findWayMap;
	findWayMap = NULL;

	if (dxFont != NULL) dxFont->Release();
	dxFont = NULL;
}

void Draw()
{
	mapSurface->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(191, 204, 156), 1.0f, 0);
	mapSurface->BeginScene();

  /*mapSurface->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	mapSurface->SetRenderState(D3DRS_ALPHATESTENABLE, true);
	mapSurface->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	mapSurface->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);*/

	for (int i = 0; i < mapHeight; i++)
		for (int j = 0; j < mapWidth; j++)
		{
			quad.Set(10 + j * mapTiles[mtFree]->GetWidth(), 10 + i * mapTiles[mtFree]->GetHeight(), 10 + (j + 1) * mapTiles[mtFree]->GetWidth(), 10 + i * mapTiles[mtFree]->GetHeight(), 10 + (j + 1) * mapTiles[mtFree]->GetWidth(), 10 + (i + 1) * mapTiles[mtFree]->GetHeight(), 10 + j * mapTiles[mtFree]->GetWidth(), 10 + (i + 1) * mapTiles[mtFree]->GetHeight());

			if (map->map[i][j] == mtFree) mapSurface->SetTexture(0, mapTiles[mtFree]->GetTexture());
			else if (map->map[i][j] == mtWall) mapSurface->SetTexture(0, mapTiles[mtWall]->GetTexture());
			else if (map->map[i][j] == mtWay) mapSurface->SetTexture(0, mapTiles[mtWay]->GetTexture());
			else if (map->map[i][j] == mtStart) mapSurface->SetTexture(0, mapTiles[mtStart]->GetTexture());
			else if (map->map[i][j] == mtEnd) mapSurface->SetTexture(0, mapTiles[mtEnd]->GetTexture());
			
			mapSurface->SetFVF(D3DFVF_VERTEX2D);
			mapSurface->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, &quad.vertices, sizeof(SVertex2D));
		}

	mapSurface->EndScene();
	mapSurface->Present(NULL, NULL, NULL, NULL);

	/*mapSurface->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
	mapSurface->SetRenderState(D3DRS_ALPHATESTENABLE, false);*/
}

LRESULT CALLBACK WndProc(HWND hwnd , UINT uMsg , WPARAM wParam , LPARAM lParam)	
{
	PAINTSTRUCT ps ;

	switch (uMsg)
	{
		case WM_CLOSE : // po otrzymaniu tego komunikatu aplikacji powinna zakoczy dziaanie 
		{
			PostQuitMessage(0);		
			return 0 ;
		}
		case WM_KEYDOWN :	// nacisnito jakis przycisk?
		{
			// parametr wParam zawiera kod znaku, ktry zosta wciniety; w naszej tablicy 
			// zaznaczamy, e dany przycisk jest wcinity
			keys[wParam] = true ;
			return 0 ;
		}
		case WM_KEYUP :		// klawisz zosta zwolniony?
		{
			keys[wParam] = false ;	// aktualizujemy stan klawisza w tablicy
			return 0 ;
		}
		case WM_PAINT :
		{
			BeginPaint(hwnd , &ps) ;
			EndPaint(hwnd , &ps) ;
			return 0 ;
		}
	}

	// wywoujemy domyln funkcj obsugi okna
	return DefWindowProc(hwnd , uMsg , wParam , lParam) ;
}

int WINAPI WinMain(HINSTANCE hInstance , HINSTANCE hPrevInstance,	LPSTR lpCmdLine , int nCmdShow)						
{
	MSG msg ;						// komunikat
	BOOL done = false ; // zmienna warunkujca zakoczenie aplikacji
	WNDCLASSEX wc ;			// klasa okna

	hInst = hInstance ;

	wc.cbSize = sizeof(WNDCLASSEX) ;		// rozmiar struktury
	wc.style= CS_HREDRAW | CS_VREDRAW ;	// odrywanie na skutek zmiany szer/wys okna
	wc.lpfnWndProc = WndProc ;					// adres precdury okienkowej
	wc.cbClsExtra = 0 ;	
	wc.cbWndExtra = 0 ;
	wc.hInstance = hInst ;							// instancja aplikacji
	wc.hIcon = LoadIcon(NULL , IDI_APPLICATION) ;		// wczytujemy domyln ikon
	wc.hIconSm = LoadIcon(NULL , IDI_WINLOGO) ;	// oraz ma ikon - logo Windows
	wc.hCursor = LoadCursor(NULL , IDC_ARROW) ;	// kursor- standardowa strzaka
	wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH) ;	// biae to
	wc.lpszMenuName = NULL ;		// bez menu
	wc.lpszClassName = "IskarowaKlasa" ;	// nazwa klasy okien	

	// rejestrujemy klas okna; jeeli ta operacja si nie powiedzie,
	// to wywietlamy odpowiedni komunikat, a program koczy dziaanie
	if (!RegisterClassEx(&wc))
	{
		MessageBox(NULL , "Nie powioda si prba inicjalizacji klasy okna." , "Bd!" , MB_OK | MB_ICONEXCLAMATION) ;
		return false ;
	}
	
	// teraz czas na utworzenie okna naszej klasy;
	// jeeli prba nie powiedzie si, wywietlamy komunikat, a aplikacja koczy dziaanie
	if (!(hwnd = CreateWindowEx(NULL ,			  
														"IskarowaKlasa" ,						
														"Wyszukiwanie drogi by Iskar 21.07.2007 pk@unit1.pl www.unit1.pl" ,						
														WS_OVERLAPPEDWINDOW |
														WS_VISIBLE |		
														WS_SYSMENU ,						
														0 , 0 ,								
														width ,	
														height ,	
														NULL ,								
														NULL ,								
														hInstance ,					
														NULL)))							

	{
		MessageBox(NULL , "Nie powioda si prba utworzenia okna." , "Bd!" , MB_OK | MB_ICONEXCLAMATION) ;
		return false ;
	}

	InitD3D();
	CreateObjects();

	// za pomoc QueryPerformanceCounter wyznaczamy dugo trwania wyszukiwania drogi
	__int64 t1,t2,f;

	QueryPerformanceFrequency((LARGE_INTEGER*)&f);
	QueryPerformanceCounter((LARGE_INTEGER*)&t1);

	findWayMap->FindWay(map->startPos, map->endPos, &way, map->map);
	
	QueryPerformanceCounter((LARGE_INTEGER*)&t2);
	MessageBox(0, ("Czas wyszukiwania: " + FloatToStr((t2 - t1) * 1000 / f)).c_str(), "", 0);
	
	for (int i = 0; i < way.size(); i++)
		map->map[way[i].Y][way[i].X] = mtWay;
	//
	

	ShowWindow(hwnd , nCmdShow) ;	// pokazujemy okno
	UpdateWindow(hwnd) ;

	while(!done)	// ptla obsugi komunikatw
	{
		if (PeekMessage(&msg , NULL , 0 , 0 , PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				done = true ;
			else
			{
				TranslateMessage(&msg) ;
				DispatchMessage(&msg) ;
			}
		}
		else
		{
			Draw();

			if (keys[VK_ESCAPE])
				done = TRUE ;
		}
	}

	ReleaseObjects();
	FreeD3D();

	return (msg.wParam) ;
}